# Stubs for zope.schema (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from zope.schema._bootstrapinterfaces import NO_VALUE as NO_VALUE
from zope.schema._field import ASCII as ASCII, ASCIILine as ASCIILine, Bool as Bool, Bytes as Bytes, BytesLine as BytesLine, Choice as Choice, Collection as Collection, Complex as Complex, Container as Container, Date as Date, Datetime as Datetime, Decimal as Decimal, Dict as Dict, DottedName as DottedName, Field as Field, Float as Float, FrozenSet as FrozenSet, Id as Id, Int as Int, Integral as Integral, InterfaceField as InterfaceField, Iterable as Iterable, List as List, Mapping as Mapping, MinMaxLen as MinMaxLen, MutableMapping as MutableMapping, MutableSequence as MutableSequence, NativeString as NativeString, NativeStringLine as NativeStringLine, Number as Number, Object as Object, Orderable as Orderable, Password as Password, PythonIdentifier as PythonIdentifier, Rational as Rational, Real as Real, Sequence as Sequence, Set as Set, SourceText as SourceText, Text as Text, TextLine as TextLine, Time as Time, Timedelta as Timedelta, Tuple as Tuple, URI as URI
from zope.schema._schema import getFieldNames as getFieldNames, getFieldNamesInOrder as getFieldNamesInOrder, getFields as getFields, getFieldsInOrder as getFieldsInOrder, getSchemaValidationErrors as getSchemaValidationErrors, getValidationErrors as getValidationErrors
from zope.schema.accessors import accessors as accessors
from zope.schema.interfaces import ValidationError as ValidationError
