# Stubs for zope.schema._schema (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from zope.schema._bootstrapfields import getFields as getFields

def getFieldNames(schema: Any): ...
def getFieldsInOrder(schema: Any, _field_key: Any = ...): ...
def getFieldNamesInOrder(schema: Any): ...
def getValidationErrors(schema: Any, value: Any): ...
def getSchemaValidationErrors(schema: Any, value: Any): ...
