# Stubs for zope.schema.fieldproperty (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

class FieldUpdatedEvent:
    inst: Any = ...
    field: Any = ...
    old_value: Any = ...
    new_value: Any = ...
    def __init__(self, inst: Any, field: Any, old_value: Any, new_value: Any) -> None: ...

class FieldProperty:
    def __init__(self, field: Any, name: Optional[Any] = ...) -> None: ...
    def __get__(self, inst: Any, klass: Any): ...
    def queryValue(self, inst: Any, default: Any): ...
    def __set__(self, inst: Any, value: Any) -> None: ...
    def __getattr__(self, name: Any): ...

def createFieldProperties(schema: Any, omit: Any = ...) -> None: ...

class FieldPropertyStoredThroughField:
    field: Any = ...
    def __init__(self, field: Any, name: Optional[Any] = ...) -> None: ...
    def setValue(self, inst: Any, field: Any, value: Any) -> None: ...
    def getValue(self, inst: Any, field: Any): ...
    def queryValue(self, inst: Any, field: Any, default: Any): ...
    def __getattr__(self, name: Any): ...
    def __get__(self, inst: Any, klass: Any): ...
    def __set__(self, inst: Any, value: Any): ...
